@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 1995 Paul Olav Tvete <paul@troll.no>
  * Copyright (C) 2000-2009 Stephan Kulow <coolo@kde.org>
+ * Copyright (C) 2010 Parker Coates <parker.coates@kdemail.net>
  *
  * License of original code:
  * -------------------------------------------------------------------------
@@ -36,22 +37,21 @@
 
 #include "klondike.h"
 
-#include "carddeck.h"
 #include "dealerinfo.h"
+#include "pileutils.h"
+#include "settings.h"
 #include "speeds.h"
-#include "version.h"
 #include "patsolve/klondikesolver.h"
 
-#include <KConfigGroup>
-#include <KDebug>
+#include "Shuffle"
+
 #include <KLocale>
-#include <KRandom>
 #include <KSelectAction>
 
 
-
-KlondikePile::KlondikePile( int _index, int _draw, const QString & objectName)
-    : Pile(_index, objectName), m_draw( _draw )
+KlondikePile::KlondikePile( KCardScene * cardScene, int index, int draw, const QString & objectName )
+  : PatPile( cardScene, index, objectName ),
+    m_draw( draw )
 {
 }
 
@@ -62,79 +62,73 @@ void KlondikePile::setDraws( int _draw )
 
 void KlondikePile::layoutCards( int duration )
 {
-    if ( m_cards.isEmpty() )
+    QList<KCard*> cards = this->cards();
+
+    if ( cards.isEmpty() )
         return;
 
-    qreal divx = qMin<qreal>( ( maximumSpace().width() - cardScene()->deck()->cardWidth() ) / ( 2 * spread().width() * cardScene()->deck()->cardWidth() ), 1.0 );
+    qreal cardWidth = cards.first()->boundingRect().width();
+
+    qreal divx = qMin<qreal>( ((availableSpace().width() - 1) * cardWidth ) / ( 2 * spread().width() * cardWidth ), 1.0 );
 
     QPointF cardPos = pos();
     int z = zValue();
-    for ( int i = 0; i < m_cards.size(); ++i )
+    for ( int i = 0; i < cards.size(); ++i )
     {
         ++z;
-        if ( i < m_cards.size() - m_draw )
-        {
-            m_cards[i]->setZValue( z );
-            m_cards[i]->animate( cardPos, z, 1, 0, m_cards[i]->realFace(), false, duration );
-        }
-        else
-        {
-            m_cards[i]->animate( cardPos, z, 1, 0, m_cards[i]->realFace(), false, duration );
-            cardPos.rx() += divx * spread().width() * cardScene()->deck()->cardWidth();
-        }
+        cards[i]->setZValue( z );
+        cards[i]->animate( cardPos, z, 0, cards[i]->isFaceUp(), false, duration );
+        if ( i >= cards.size() - m_draw )
+            cardPos.rx() += divx * spread().width() * cardWidth;
     }
 }
 
-Klondike::Klondike()
-    : DealerScene( )
+void Klondike::initialize()
 {
     // The units of the follwoing constants are pixels
     const qreal hspacing = 1.0 / 6 + 0.02; // horizontal spacing between card piles
     const qreal vspacing = 1.0 / 4; // vertical spacing between card piles
 
-    setDeck( new CardDeck() );
+    static_cast<KStandardCardDeck*>( deck() )->setDeckContents();
 
-    talon = new Pile( 0, "talon" );
-    talon->setPilePos(0, 0);
-    talon->setAddFlags(Pile::disallow);
-    connect(talon, SIGNAL(clicked(Card*)), SLOT(newCards()));
+    EasyRules = Settings::klondikeIsDrawOne();
+
+    talon = new PatPile( this, 0, "talon" );
+    talon->setPileRole(PatPile::Stock);
+    talon->setLayoutPos(0, 0);
     // Give the talon a low Z value to keep it out of the way during there
     // deal animation.
     talon->setZValue( -52 );
-    addPile(talon);
+    talon->setKeyboardSelectHint( KCardPile::NeverFocus );
+    talon->setKeyboardDropHint( KCardPile::NeverFocus );
+    connect( talon, SIGNAL(clicked(KCard*)), SLOT(drawDealRowOrRedeal()) );
 
-    KConfigGroup cg(KGlobal::config(), settings_group );
-    EasyRules = cg.readEntry( "KlondikeEasy", true);
-
-    pile = new KlondikePile( 13, EasyRules ? 1 : 3, "pile" );
-    pile->setObjectName( "pile" );
-    pile->setReservedSpace( QSizeF( 1.9, 1.0 ) );
-    pile->setPilePos(1.0 + hspacing, 0);
+    pile = new KlondikePile( this, 13, EasyRules ? 1 : 3, "pile" );
+    pile->setPileRole(PatPile::Waste);
+    pile->setReservedSpace( 0, 0, 1.9, 1.0 );
+    pile->setLayoutPos(1.0 + hspacing, 0);
     pile->setSpread( 0.33, 0 );
-    pile->setAddFlags( Pile::disallow );
-    pile->setRemoveFlags(Pile::Default);
-    addPile(pile);
+    pile->setKeyboardSelectHint( KCardPile::ForceFocusTop );
+    pile->setKeyboardDropHint( KCardPile::NeverFocus );
 
-    for( int i = 0; i < 7; i++ )
+    for( int i = 0; i < 7; ++i )
     {
-        play[ i ] = new Pile( i + 5, QString( "play%1" ).arg( i ));
-        play[i]->setPilePos((1.0 + hspacing) * i, 1.0 + vspacing);
-        play[i]->setAddType(Pile::KlondikeStore);
-        play[i]->setRemoveFlags(Pile::several | Pile::autoTurnTop | Pile::wholeColumn);
-        play[i]->setReservedSpace( QSizeF( 1.0, 1.0 + play[i]->spread().height() * 7 ) );
-        addPile(play[i]);
+        play[i] = new PatPile( this, i + 5, QString( "play%1" ).arg( i ));
+        play[i]->setPileRole(PatPile::Tableau);
+        play[i]->setLayoutPos((1.0 + hspacing) * i, 1.0 + vspacing);
+        play[i]->setAutoTurnTop(true);
+        play[i]->setReservedSpace( 0, 0, 1, 1 + play[i]->spread().height() * 7 );
+        play[i]->setKeyboardSelectHint( KCardPile::AutoFocusDeepestFaceUp );
+        play[i]->setKeyboardDropHint( KCardPile::AutoFocusTop );
     }
 
-    for( int i = 0; i < 4; i++ )
+    for( int i = 0; i < 4; ++i )
     {
-        target[ i ] = new Pile( i + 1, QString( "target%1" ).arg( i ) );
-        target[i]->setPilePos((3 + i) * (1.0 + hspacing), 0);
-        target[i]->setAddType(Pile::KlondikeTarget);
-        if (EasyRules)
-            target[i]->setRemoveFlags(Pile::Default);
-        else
-            target[i]->setRemoveType(Pile::KlondikeTarget);
-        addPile(target[i]);
+        target[i] = new PatPile( this, i + 1, QString( "target%1" ).arg( i ) );
+        target[i]->setPileRole(PatPile::Foundation);
+        target[i]->setLayoutPos((3 + i) * (1.0 + hspacing), 0);
+        target[i]->setKeyboardSelectHint( KCardPile::ForceFocusTop );
+        target[i]->setKeyboardDropHint( KCardPile::ForceFocusTop );
     }
 
     setActions(DealerScene::Hint | DealerScene::Demo | DealerScene::Draw);
@@ -148,72 +142,93 @@ Klondike::Klondike()
     connect( options, SIGNAL(triggered(int)), SLOT(gameTypeChanged()) );
 }
 
+bool Klondike::checkAdd(const PatPile * pile, const QList<KCard*> & oldCards, const QList<KCard*> & newCards) const
+{
+    switch (pile->pileRole())
+    {
+    case PatPile::Tableau:
+        return checkAddAlternateColorDescendingFromKing(oldCards, newCards);
+    case PatPile::Foundation:
+        return checkAddSameSuitAscendingFromAce(oldCards, newCards);
+    case PatPile::Waste:
+    case PatPile::Stock:
+    default:
+        return false;
+    }
+}
+
+bool Klondike::checkRemove(const PatPile * pile, const QList<KCard*> & cards) const
+{
+    switch (pile->pileRole())
+    {
+    case PatPile::Tableau:
+        return isAlternateColorDescending(cards);
+    case PatPile::Foundation:
+        return EasyRules && cards.first() == pile->top();
+    case PatPile::Waste:
+        return cards.first() == pile->top();
+    case PatPile::Stock:
+    default:
+        return false;
+    }
+}
+
+
 QList<QAction*> Klondike::configActions() const
 {
     return QList<QAction*>() << options;
 }
 
-Card *Klondike::newCards()
+
+bool Klondike::newCards()
 {
-    if ( deck()->hasAnimatedCards() )
-    {
-        if ( pile->top() )
-            return pile->top();
-        for (int i = 0; i < 7; ++i)
-            if (play[i]->top())
-                return play[i]->top();
-    }
+    if ( talon->isEmpty() && pile->count() <= 1 )
+        return false;
 
     if ( talon->isEmpty() )
     {
         // Move the cards from the pile back to the deck
-        while ( !pile->isEmpty() )
-        {
-            Card *c = pile->top();
-            c->completeAnimation();
-            c->turn( false );
-            talon->add(c);
-        }
-        talon->relayoutCards();
+        flipCardsToPile( pile->cards(), talon, DURATION_MOVE );
 
         redealt = true;
     }
     else
     {
-        QList<Card*> flippedCards;
-        for (int flipped = 0; flipped < pile->draw() && !talon->isEmpty(); ++flipped)
-        {
-            pile->add( talon->top() );
-            flippedCards << pile->top();
-        }
-        pile->relayoutCards();
-        int flipped = 0;
-        foreach ( Card *c, flippedCards )
-        {
-            ++flipped;
-            c->completeAnimation();
-            QPointF destPos = c->realPos();
-            c->setPos( talon->pos() );
-            c->flipTo( destPos, DURATION_FLIP * ( 1 + flipped / 6.0) );
-        }
+        int depth = qMax( 0, talon->count() - pile->draw() );
+        QList<KCard*> cards = talon->topCardsDownTo( talon->at( depth ) );
+        flipCardsToPile( cards, pile, DURATION_MOVE );
+        setKeyboardFocus( pile->top() );
     }
 
-    onGameStateAlteredByUser();
-    if ( talon->isEmpty() && pile->cardsLeft() <= 1 )
+    if ( talon->isEmpty() && pile->count() <= 1 )
        emit newCardsPossible( false );
 
     // we need to look that many steps in the future to see if we can lose
-    setNeededFutureMoves( talon->cardsLeft() + pile->cardsLeft() );
+    setNeededFutureMoves( talon->count() + pile->count() );
 
-    return pile->top() ? pile->top() : talon->top();
+    return true;
 }
 
+
 void Klondike::restart()
 {
-    deck()->returnAllCards();
-    deck()->shuffle( gameNumber() );
     redealt = false;
-    deal();
+
+    QList<KCard*> cards = shuffled( deck()->cards(), gameNumber() );
+
+    for( int round = 0; round < 7; ++round )
+        for ( int i = round; i < 7; ++i )
+            addCardForDeal( play[i], cards.takeLast(), (i == round), talon->pos());
+
+    while ( !cards.isEmpty() )
+    {
+        KCard * c = cards.takeFirst();
+        c->setPos( talon->pos() );
+        c->setFaceUp( false );
+        talon->add( c );
+    }
+
+    startDealAnimation();
 }
 
 void Klondike::gameTypeChanged()
@@ -238,7 +253,7 @@ QString Klondike::getGameState()
     // getGameState() is called every time a card is moved, so we use it to
     // check if there are any cards left to deal. There might be a more elegant
     // to do this, though.
-    emit newCardsPossible( !talon->isEmpty() || pile->cardsLeft() > 1 );
+    emit newCardsPossible( !talon->isEmpty() || pile->count() > 1 );
     return QString();
 }
 
@@ -261,32 +276,20 @@ void Klondike::setEasy( bool _EasyRules )
 
         int drawNumber = EasyRules ? 1 : 3;
         pile->setDraws( drawNumber );
+
+        KCardPile::KeyboardFocusHint hint = EasyRules
+                                          ? KCardPile::ForceFocusTop
+                                          : KCardPile::NeverFocus;
+        for( int i = 0; i < 4; ++i )
+            target[i]->setKeyboardSelectHint( hint );
+
         setSolver( new KlondikeSolver( this, drawNumber ) );
 
-        for( int i = 0; i < 4; i++ ) {
-            if (EasyRules) // change default
-                target[i]->setRemoveFlags(Pile::Default);
-            else
-                target[i]->setRemoveType(Pile::KlondikeTarget);
-        }
-
-        KConfigGroup cg(KGlobal::config(), settings_group );
-        cg.writeEntry( "KlondikeEasy", EasyRules);
-        cg.sync();
+        Settings::setKlondikeIsDrawOne( EasyRules );
     }
 }
 
 
-void Klondike::deal() {
-    for(int round=0; round < 7; round++)
-        for (int i = round; i < 7; i++ )
-            addCardForDeal( play[i], deck()->takeCard(), (i == round), talon->pos());
-
-    deck()->takeAllCards(talon);
-
-    startDealAnimation();
-}
-
 bool Klondike::drop()
 {
     bool pileempty = pile->isEmpty();
