@@ -1,8 +1,26 @@
-/* Common routines & arrays. */
+/*
+ * Copyright (C) 1998-2002 Tom Holroyd <tomh@kurage.nimh.nih.gov>
+ * Copyright (C) 2006-2009 Stephan Kulow <coolo@kde.org>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of 
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "patsolve.h"
 
-#include "../pile.h"
+#include "../patpile.h"
+
+#include "KStandardCardDeck"
 
 #include <KDebug>
 
@@ -14,6 +32,10 @@
 #include <sys/types.h>
 
 
+#ifdef ERR
+#undef ERR
+#endif
+
 long all_moves = 0;
 
 /* This is a 32 bit FNV hash.  For more information, see
@@ -86,7 +108,7 @@ bool Solver::recursive(POSITION *parent)
 
     if (alln == 0) {
         if ( isWon() ) {
-            Status = WIN;
+            Status = SolutionExists;
             Q_ASSERT(parent); // it just is never called with a won game
             win(parent);
             return true;
@@ -115,7 +137,7 @@ bool Solver::recursive(POSITION *parent)
         return false;
     }
     MOVE *mp = mp0;
-    for (i = 0; i < alln; i++) {
+    for (i = 0; i < alln; ++i) {
         if (Possible[i].card_index != -1) {
             *mp = Possible[i];      /* struct copy */
             mp++;
@@ -125,7 +147,7 @@ bool Solver::recursive(POSITION *parent)
     ++alln;
 
     bool fit = false;
-    for (mp = mp0; mp->card_index != -1; mp++) {
+    for (mp = mp0; mp->card_index != -1; ++mp) {
 
         int depth = 0;
         if (parent != NULL)
@@ -172,7 +194,7 @@ bool Solver::recursive(POSITION *parent)
 #endif
         Total_positions++;
         if ( Total_positions % 10000 == 0 )
-            kDebug(11111) << "positions" << Total_positions;
+            kDebug() << "positions" << Total_positions;
 
         pos->move = *mp;                 /* struct copy */
         pos->cluster = 0;
@@ -181,7 +203,6 @@ bool Solver::recursive(POSITION *parent)
 
         bool ret = recursive(pos);
         fit |= ret;
-        //kDebug(11111) << "ret" << ret << " " << depth;
         undo_move(mp);
         mm->give_back_block( (quint8 *)pos->node );
         if ( ret )
@@ -252,20 +273,20 @@ MOVE *Solver::get_moves(int *nmoves)
 	*nmoves = n;
 	i = 0;
 	if (a || numout == 0) {
-		for (i = 0; i < alln; i++) {
+		for (i = 0; i < alln; ++i) {
 			if (Possible[i].card_index != -1) {
 				*mp = Possible[i];      /* struct copy */
 				mp++;
 			}
 		}
 	} else {
-		for (i = numout; i < alln; i++) {
+		for (i = numout; i < alln; ++i) {
 			if (Possible[i].card_index != -1) {
 				*mp = Possible[i];      /* struct copy */
 				mp++;
 			}
 		}
-		for (i = 0; i < numout; i++) {
+		for (i = 0; i < numout; ++i) {
 			if (Possible[i].card_index != -1) {
 				*mp = Possible[i];      /* struct copy */
 				mp++;
@@ -346,7 +367,7 @@ TREE *Solver::pack_position(void)
 
 	p = mm->new_from_block(Treebytes);
 	if (p == NULL) {
-                Status = FAIL;
+                Status = UnableToDetermineSolvability;
 		return NULL;
 	}
 	node = (TREE *)p;
@@ -362,7 +383,7 @@ TREE *Solver::pack_position(void)
 
 	k = 0;
         quint16 *p2 = ( quint16* ) p;
-	for (w = 0; w < m_number_piles; w++) {
+	for (w = 0; w < m_number_piles; ++w) {
 		j = Wpilenum[w];
                 if ( j < 0 )
                 {
@@ -458,7 +479,7 @@ void Solver::win(POSITION *pos)
 
     mpp0 = new_array(MOVE *, nmoves);
     if (mpp0 == NULL) {
-        Status = FAIL;
+        Status = UnableToDetermineSolvability;
         return; /* how sad, so close... */
     }
     mpp = mpp0 + nmoves - 1;
@@ -466,7 +487,7 @@ void Solver::win(POSITION *pos)
         *mpp-- = &p->move;
     }
 
-    for (i = 0, mpp = mpp0; i < nmoves; i++, mpp++)
+    for (i = 0, mpp = mpp0; i < nmoves; ++i, ++mpp)
         winMoves.append( **mpp );
 
     MemoryManager::free_array(mpp0, nmoves);
@@ -540,17 +561,17 @@ int Solver::get_pilenum(int w)
 
 	if (l == NULL) {
 		if (Pilenum >= NPILES ) {
-                        Status = FAIL;
+                        Status = UnableToDetermineSolvability;
 			return -1;
 		}
 		l = mm_allocate(BUCKETLIST);
 		if (l == NULL) {
-                        Status = FAIL;
+                        Status = UnableToDetermineSolvability;
 			return -1;
 		}
 		l->pile = new_array(quint8, Wlen[w] + 1);
 		if (l->pile == NULL) {
-                    Status = FAIL;
+                    Status = UnableToDetermineSolvability;
                     MemoryManager::free_ptr(l);
                     return -1;
 		}
@@ -573,7 +594,7 @@ int Solver::get_pilenum(int w)
 #if 0
 if (w < 4) {
         fprintf( stderr, "get_pile_num %d ", l->pilenum );
-        for (int i = 0; i < Wlen[w]; i++) {
+        for (int i = 0; i < Wlen[w]; ++i) {
             printcard(W[w][i], stderr);
         }
         fprintf( stderr, "\n" );
@@ -614,7 +635,7 @@ void Solver::doit()
 
 	/* Init the queues. */
 
-	for (i = 0; i < NQUEUES; i++) {
+	for (i = 0; i < NQUEUES; ++i) {
 		Qhead[i] = NULL;
 	}
 	Maxq = 0;
@@ -628,7 +649,7 @@ void Solver::doit()
 	pos = new_position(NULL, &m);
 	if ( pos == NULL )
         {
-            Status = FAIL;
+            Status = UnableToDetermineSolvability;
             return;
         }
 	queue_position(pos, 0);
@@ -660,19 +681,22 @@ bool Solver::solve(POSITION *parent)
 	but always return false from any position.  This enables the cleanup
 	of the move stack and eventual destruction of the position store. */
 
-	if (Status != NOSOL) {
+	if (Status != NoSolutionExists) {
 		return false;
 	}
 
-        if ( m_shouldEnd )
         {
-            Status = QUIT;
-            return false;
+            QMutexLocker lock( &endMutex );
+            if ( m_shouldEnd )
+            {
+                Status = SearchAborted;
+                return false;
+            }
         }
 
         if ( max_positions != -1 && Total_positions > ( unsigned long )max_positions )
         {
-            Status = MLIMIT;
+            Status = MemoryLimitReached;
             return false;
         }
 
@@ -681,7 +705,7 @@ bool Solver::solve(POSITION *parent)
 
 	if ((mp0 = get_moves(&nmoves)) == NULL) {
             if ( isWon() ) {
-                Status = WIN;
+                Status = SolutionExists;
                 win( parent );
             }
             return false;
@@ -690,7 +714,7 @@ bool Solver::solve(POSITION *parent)
         if ( parent->depth == 0 )
         {
             Q_ASSERT( firstMoves.count() == 0 );
-            for (int j = 0; j < nmoves; j++)
+            for (int j = 0; j < nmoves; ++j)
                 firstMoves.append( Possible[j] );
         }
 
@@ -699,7 +723,7 @@ bool Solver::solve(POSITION *parent)
 	/* Make each move and either solve or queue the result. */
 
 	q = false;
-	for (i = 0, mp = mp0; i < nmoves; i++, mp++) {
+	for (i = 0, mp = mp0; i < nmoves; ++i, ++mp) {
 		make_move(mp);
 
 		/* Calculate indices for the new piles. */
@@ -780,8 +804,8 @@ void Solver::queue_position(POSITION *pos, int pri)
 
         int nout = getOuts();
 
-        static double Yparam[] = { 0.0032, 0.32, -3.0 };
-	double x = (Yparam[0] * nout + Yparam[1]) * nout + Yparam[2];
+        static qreal Yparam[] = { 0.0032, 0.32, -3.0 };
+	qreal x = (Yparam[0] * nout + Yparam[1]) * nout + Yparam[2];
 	pri += (int)floor(x + .5);
 
 	if (pri < 0) {
@@ -880,7 +904,7 @@ Solver::~Solver()
 {
     delete mm;
 
-    for ( int i = 0; i < m_number_piles; i++ )
+    for ( int i = 0; i < m_number_piles; ++i )
     {
         delete [] W[i];
     }
@@ -903,7 +927,7 @@ void Solver::init()
 
     /* Reset stats. */
 
-    Status = NOSOL;
+    Status = NoSolutionExists;
     Total_positions = 0;
     Total_generated = 0;
     depth_sum = 0;
@@ -918,7 +942,7 @@ void Solver::free()
 }
 
 
-Solver::statuscode Solver::patsolve( int _max_positions )
+Solver::ExitStatus Solver::patsolve( int _max_positions )
 {
     max_positions = _max_positions;
 
@@ -928,7 +952,7 @@ Solver::statuscode Solver::patsolve( int _max_positions )
     /* Go to it. */
     doit();
 
-    if ( Status == QUIT ) // thread quit
+    if ( Status == SearchAborted ) // thread quit
     {
         firstMoves.clear();
         winMoves.clear();
@@ -952,7 +976,7 @@ void Solver::setNumberPiles( int p )
 
     /* Work arrays. */
     W = new card_t*[m_number_piles];
-    for ( int i = 0; i < m_number_piles; i++ )
+    for ( int i = 0; i < m_number_piles; ++i )
     {
         W[i] = new card_t[84];
         memset( W[i], 0, sizeof( card_t ) * 84 );
@@ -976,23 +1000,23 @@ int Solver::translateSuit( int s )
     return suit;
 }
 
-int Solver::translate_pile(const Pile *pile, card_t *w, int size)
+int Solver::translate_pile(const KCardPile *pile, card_t *w, int size)
 {
     Q_UNUSED( size );
-        Q_ASSERT( pile->cardsLeft() <= size );
+        Q_ASSERT( pile->count() <= size );
 
         card_t rank, suit;
 
 	rank = suit = NONE;
-        for ( int i = 0; i < pile->cardsLeft(); ++i )
+        for ( int i = 0; i < pile->count(); ++i )
         {
-            Card *c = pile->at( i );
-            *w =  + translateSuit( c->suit() ) + c->rank();
-            if ( !c->realFace() )
+            KCard *c = pile->at( i );
+            *w =  + translateSuit( getSuit(c) ) + getRank(c);
+            if ( !c->isFaceUp() )
                 *w += 1 << 7;
             w++;
 	}
-	return pile->cardsLeft();
+	return pile->count();
 }
 
 /* Insert key into the tree unless it's already there.  Return true if
@@ -1063,7 +1087,7 @@ POSITION *Solver::new_position(POSITION *parent, MOVE *m)
 	} else {
 		p = mm->new_from_block(Posbytes);
 		if (p == NULL) {
-                        Status = FAIL;
+                        Status = UnableToDetermineSolvability;
 			return NULL;
 		}
 	}
@@ -1079,14 +1103,14 @@ POSITION *Solver::new_position(POSITION *parent, MOVE *m)
 #if 0
         QString dummy;
         quint16 *t = ( quint16* )( ( char* )node + sizeof( TREE ) );
-        for ( int i = 0; i < m_number_piles; i++ )
+        for ( int i = 0; i < m_number_piles; ++i )
         {
             QString s = "      " + QString( "%1" ).arg( ( int )t[i] );
             dummy += s.right( 5 );
         }
         if ( Total_positions % 1000 == 1000 )
             print_layout();
-        kDebug(11111) << "new" << dummy;
+        kDebug() << "new" << dummy;
 #endif
 	p += sizeof(POSITION);
 	return pos;
