@@ -20,21 +20,25 @@
 #ifndef __KCHATDIALOG_H__
 #define __KCHATDIALOG_H__
 
-#include <kdialogbase.h>
-#include <kdemacros.h>
+#include <libkdegames_export.h>
+
+#include <kdialog.h>
 
 class KChatBase;
 
 class KChatDialogPrivate;
 
-class KDE_EXPORT KChatDialog : public KDialogBase
+/**
+ * \class KChatDialog kchatdialog.h <KChatDialog>
+ * */
+class KDEGAMES_EXPORT KChatDialog : public KDialog
 {
 	Q_OBJECT
 public:
 	/**
 	 * Construct a KChatDialog widget
 	 **/
-	KChatDialog(QWidget* parent, bool modal = false);
+	explicit KChatDialog(QWidget* parent, bool modal = false);
 
 	/**
 	 * Construct a KChatDialog widget which automatically configures the
@@ -74,7 +78,7 @@ public:
 	 * Set the widget that will be configured by the dialog. Use this if you
 	 * don't want to configure the widget yourself.
 	 * @param widget The chat widget that shall be configured
-	 * @param Whether you want to have the current @ref KChatBase fonts as
+	 * @param applyFonts Whether you want to have the current @ref KChatBase fonts as
 	 * defaults in the dialog
 	 **/
 	void plugChatWidget(KChatBase* widget, bool applyFonts = true);
@@ -93,7 +97,7 @@ public:
 	 **/
 	int maxMessages() const;
 
-protected slots:
+protected Q_SLOTS:
 	void slotGetNameFont();
 	void slotGetTextFont();
 	void slotGetSystemNameFont();
@@ -113,7 +117,7 @@ private:
 	void init();
 
 private:
-	KChatDialogPrivate* d;
+	KChatDialogPrivate* const d;
 };
 
 #endif
